# Installation
> `npm install --save @types/koa-compress`

# Summary
This package contains type definitions for koa-compress (https://github.com/koajs/compress).

# Details
Files were exported from https://github.com/DefinitelyTyped/DefinitelyTyped/tree/master/types/koa-compress.
## [index.d.ts](https://github.com/DefinitelyTyped/DefinitelyTyped/tree/master/types/koa-compress/index.d.ts)
````ts
// Type definitions for koa-compress v4.x
// Project: https://github.com/koajs/compress
// Definitions by: Jerry Chin <https://github.com/hellopao>
//                 Joel Gallant <https://github.com/joelgallant>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped
// TypeScript Version: 2.3

/* =================== USAGE ===================

    import compress = require("koa-compress");
    var Koa = require('koa');

    var app = new Koa();
    app.use(compress());

 =============================================== */
/// <reference types="node" />
/// <reference types="koa" />

import * as Koa from "koa";
import * as zlib from "zlib";

/**
 * Compress middleware for Koa
 */
declare function koaCompress(options?: koaCompress.CompressOptions): Koa.Middleware;

declare namespace koaCompress {
    export interface CompressOptions {
        /**
         * An optional function that checks the response content type to decide whether to compress. By default, it uses compressible.
         */
        filter?: ((mimeType: string) => boolean) | undefined;

        /**
         * Minimum response size in bytes to compress. Default 1024 bytes or 1kb.
         */
        threshold?: number | string | undefined;

        /**
         * An optional string, which specifies what encoders to use for requests
         * without Accept-Encoding. Default: 'idenity'.
         */
        defaultEncoding?: string | undefined

        /**
         * Options for brotli compression.
         */
        br?: zlib.BrotliOptions | false | undefined;

        /**
         * Options for gzip compression.
         */
        gzip?: zlib.ZlibOptions | false | undefined;

        /**
         * Options for deflate compression.
         */
        deflate?: zlib.ZlibOptions | false | undefined;
    }
}

export = koaCompress;

````

### Additional Details
 * Last updated: Fri, 09 Jul 2021 02:32:32 GMT
 * Dependencies: [@types/koa](https://npmjs.com/package/@types/koa), [@types/node](https://npmjs.com/package/@types/node)
 * Global values: none

# Credits
These definitions were written by [Jerry Chin](https://github.com/hellopao), and [Joel Gallant](https://github.com/joelgallant).
